#include <assert.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bignum.h"
#include "util.h"

int
bnum_mul(struct bignum *c, const struct bignum *a, const struct bignum *b)
{
	unsigned int ok, cc;
	int i, j, k;
	int a_val, b_val;
	int reslen;

	assert(c != NULL && a != NULL && b != NULL);

	c->sign = a->sign ^ b->sign;

	reslen = a->len + b->len;

	ok = bnum_ensure_len(c, reslen);
	bnum_clear(c);

	if (!ok)
		return BIGNUM_NOMEM;
	c->len = reslen;

	memset(c->val, 0, c->len);
	cc = 0;
	k = 0;
	for (i = 0; i <= reslen-1; i++) {

		for (j = 0; j <= i; j++) {

			a_val = (j <= a->top) ? a->val[j] : 0;
			b_val = (i - j <= b->top) ? b->val[i-j] : 0;

			cc = a_val * b_val + c->val[i];

			c->val[i] = cc % BIGNUM_BASE;

			cc = cc>>16;					/* cc = cc/BASE analog. */

			while ( cc != 0 ) {				/* Sum tail with higher digits. */
				k++;
				cc += c->val[i+k];
				c->val[i + k] = cc % BIGNUM_BASE;
				cc = cc>>16;
			}
			k = 0;
		}
	}

	c->top = i-1;
	while ( c->val[c->top] == 0) {
		if (c->top >=0)
			c->top--;
		else
			break;
	}
	return 0;
}
